@@ -6,9 +6,9 @@
 import os
 import sys
 
-from tests.coveragetest import CoverageTest
-
 from unittest_mixins import change_dir    # pylint: disable=unused-import
+
+from tests.coveragetest import CoverageTest
 from tests.test_farm import clean
 # Import helpers, eventually test_farm.py will go away.
 from tests.test_farm import (       # pylint: disable=unused-import
@@ -38,5 +38,5 @@ class CoverageGoldTest(CoverageTest):
         # beginning of the test.
         clean(the_dir)
 
-        if not os.environ.get("COVERAGE_KEEP_OUTPUT"):      # pragma: partial
+        if not os.environ.get("COVERAGE_KEEP_OUTPUT"):      # pragma: part covered
             self.addCleanup(clean, the_dir)
