@@ -295,18 +295,20 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
     """Test the behavior when measuring unparsable files."""
 
     def test_dotpy_not_python(self):
+        self.make_file("main.py", "import innocuous")
         self.make_file("innocuous.py", "a = 1")
         cov = coverage.Coverage()
-        self.start_import_stop(cov, "innocuous")
+        self.start_import_stop(cov, "main")
         self.make_file("innocuous.py", "<h1>This isn't python!</h1>")
         msg = "Couldn't parse '.*innocuous.py' as Python source: .* at line 1"
         with self.assertRaisesRegex(NotPython, msg):
             cov.html_report()
 
     def test_dotpy_not_python_ignored(self):
+        self.make_file("main.py", "import innocuous")
         self.make_file("innocuous.py", "a = 2")
         cov = coverage.Coverage()
-        self.start_import_stop(cov, "innocuous")
+        self.start_import_stop(cov, "main")
         self.make_file("innocuous.py", "<h1>This isn't python!</h1>")
         cov.html_report(ignore_errors=True)
         self.assertEqual(
@@ -372,7 +374,7 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
         self.make_file("main.py", "import sub.not_ascii")
         self.make_file("sub/__init__.py")
         self.make_file("sub/not_ascii.py", """\
-            # coding: utf8
+            # coding: utf-8
             a = 1  # Isn't this great?!
             """)
         cov = coverage.Coverage()
@@ -381,7 +383,7 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
         # Create the undecodable version of the file. make_file is too helpful,
         # so get down and dirty with bytes.
         with open("sub/not_ascii.py", "wb") as f:
-            f.write(b"# coding: utf8\na = 1  # Isn't this great?\xcb!\n")
+            f.write(b"# coding: utf-8\na = 1  # Isn't this great?\xcb!\n")
 
         with open("sub/not_ascii.py", "rb") as f:
             undecodable = f.read()
